import java.util.Properties;
import javax.mail.*;
import javax.mail.internet.*;
import java.io.*;

public class RetrieveAttachment {
    public static void main(String[] args) 
    {
        if (args.length!=3) {
        System.out.println("Usage: RetrieveAttachment host username password");
            System.exit(1);
        }

        String host = args[0];
        String username = args[1];
        String password = args[2];

        try {
            Properties props = new Properties();
            Session session = Session.getDefaultInstance(props, null);
            Store store = session.getStore("pop3");
            store.connect(host, username, password);
            Folder folder = store.getFolder("INBOX");
            folder.open(Folder.READ_ONLY);
            Message messages[] = folder.getMessages();
            for (int i=0; i<messages.length; i++) {
                Message msg = messages[i];
                System.out.println(msg.getFrom()[0] + "\t" +
                     messages[i].getSubject());
                Object obj = msg.getContent();
                if (obj instanceof Multipart)
                    processMessage((Multipart)obj);
            }

            folder.close(false);
            store.close();
        }
        catch (MessagingException me) {
             System.err.println(me);
        }
        catch (IOException ioe) {
            System.err.println(ioe);
        }
    }

    private static void processMessage(Multipart multipart) throws MessagingException, IOException
    {
        for (int i=0; i<multipart.getCount(); i++) {
            Part part = multipart.getBodyPart(i);
            String disposition = part.getDisposition();
            if (disposition != null && 
                disposition.equals(Part.ATTACHMENT))
                writeFile(part.getFileName(), part.getInputStream());
        }
    }

    private static void writeFile(String fileName, InputStream in) throws IOException 
    {
        if (fileName == null)
            fileName = "attachment.dat";
        File file = new File(fileName);
        file = new File("attach",file.getName()); 

        String rootName = file.getName();
        for (int i=0; file.exists(); i++) {
            file = new File(file.getParent(),""+i+"_"+rootName);
        }
        System.out.println("Saving attachment in: "+ file.getAbsolutePath());

        BufferedOutputStream bos = new BufferedOutputStream(new FileOutputStream(file));
        BufferedInputStream bis = new BufferedInputStream(in);
        byte[] buffer = new byte[2048];
        int n;
        while ((n=bis.read(buffer,0,buffer.length)) > 0) {
            bos.write(buffer,0,n);
        }
        bos.flush();
        bos.close();
        bis.close();
    }
}
