package data;

import java.util.*;
import javax.ejb.*;
import javax.jms.*;
import javax.naming.*;

public class ApplicantMatch implements MessageDrivenBean, MessageListener
{
    private ApplicantLocalHome applicantHome;
    private JobLocalHome jobHome;
    private MatchedLocalHome matchedHome;
    private MessageDrivenContext ctx;
    private EmailApplicant emailApplicant;
    
    public void onMessage(Message message) {
        String login = null;
        if (!(message instanceof TextMessage)) {
            System.out.println("ApplicantMatch: bad message:" + message.getClass());
            return;
        }
        try {
            login = ((TextMessage)message).getText();
            System.out.println("ApplicantMatch: "+login+" - "+message.getBooleanProperty("NewApplicant"));
            if (! message.getBooleanProperty("NewApplicant")) {
                matchedHome.deleteByApplicant(login);
            }
        }
        catch (JMSException ex) {
            error ("Error getting JMS property: NewApplicant",ex);
        }
        try {
            ApplicantLocal applicant = applicantHome.findByPrimaryKey(login);
            LocationLocal applicantLocation = applicant.getLocation();
            String location = "";
            if (applicantLocation!=null)
                location = applicantLocation.getName();
            Collection skills = applicant.getSkills();
            Collection appSkills = new ArrayList();
            Iterator appIt = skills.iterator();
            while (appIt.hasNext()) {
                SkillLocal as = (SkillLocal)appIt.next();
                appSkills.add(as.getName());
            }
            emailApplicant.newMessage();
            Collection col = jobHome.findByLocation(location);
            Iterator jobsIter = col.iterator();
            while (jobsIter.hasNext()) {
                JobLocal job = (JobLocal)jobsIter.next();
                Collection jobSkills = job.getSkills();
                int skillMatch = 0;
                Iterator jobSkillIter = jobSkills.iterator();
                while (jobSkillIter.hasNext()) {
                    SkillLocal jobSkill = (SkillLocal)jobSkillIter.next();
                    if (appSkills.contains(jobSkill.getName()))
                       skillMatch++;
                }
                if (jobSkills.size() > 0 && skillMatch == 0)
                    continue;
                boolean exact = skillMatch == jobSkills.size();
                MatchedPK key = new MatchedPK(login,job.getRef(),job.getCustomer());
                try {
                    matchedHome.create(key.getApplicant(),key.getJob(),key.getCustomer(), exact);
                    emailApplicant.addJob(new JobPK(job.getRef(), job.getCustomer()));
                }
                catch (CreateException ex) {
                    System.out.println("ApplicantMatch: failed to create matched entry: "+key);
                }
            }
            emailApplicant.sendMail(applicant.getEmail());
        }
        catch (FinderException ex) {
            System.out.println("ApplicantMatch: failed to find applicant data: "+login);
        }
        catch (javax.mail.MessagingException me) {
            System.err.println("ApplicantMatch: mail error "+me);
        }
        catch (RuntimeException ex) {
            System.out.println("ApplicantMatch: "+ex);
            ex.printStackTrace();
            throw ex;
        }
    }

    // EJB methods start here

    public void setMessageDrivenContext(MessageDrivenContext ctx) {
        this.ctx = ctx;
        InitialContext ic = null;
        try {
            ic = new InitialContext();
            applicantHome = (ApplicantLocalHome)ic.lookup("java:comp/env/ejb/ApplicantLocal");
        }
        catch (NamingException ex) {
            error("Error connecting to java:comp/env/ejb/ApplicantLocal:",ex);
        }
        try {
            jobHome = (JobLocalHome)ic.lookup("java:comp/env/ejb/JobLocal");
        }
        catch (NamingException ex) {
            error("Error connecting to java:comp/env/ejb/JobLocal:",ex);
        }
        try {
            matchedHome = (MatchedLocalHome)ic.lookup("java:comp/env/ejb/MatchedLocal");
        }
        catch (NamingException ex) {
            error("Error connecting to java:comp/env/ejb/MatchedLocal:",ex);
        }
        try {
            emailApplicant = new EmailApplicant((javax.mail.Session)ic.lookup("java:comp/env/mail/James"));
        }
        catch (NamingException ex) {
            error("Error connecting to java:comp/env/mail/James:",ex);
        }
    }

    public void ejbCreate(){
    }

    public void ejbRemove(){
        applicantHome = null;
        jobHome = null;
        matchedHome = null;
    }

    private void error (String msg, Exception ex) {
        String s = "ApplicantMatch: "+msg + "\n" + ex;
        System.out.println(s);
        throw new EJBException(s,ex);
    }
}


